use actix_web::{web, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use chrono::{NaiveDate, Utc};
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::{Area, Availabillity, Event, Function, Role, User};

#[derive(Deserialize)]
pub struct CalendarQuery {
    date: Option<NaiveDate>,
}

#[derive(Template)]
#[template(path = "index.html")]
struct CalendarTemplate {
    user: User,
    date: NaiveDate,
    areas: Vec<Area>,
    events: Vec<Event>,
    availabillities: Vec<Availabillity>,
}

#[actix_web::get("/")]
async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<CalendarQuery>,
) -> impl Responder {
    let date = match query.date {
        Some(given_date) => given_date,
        None => Utc::now().date_naive(),
    };
    let areas = Area::read_all(pool.get_ref()).await.unwrap();

    let events = Event::read_by_date_including_location(pool.get_ref(), date)
        .await
        .unwrap();

    let availabillities = Availabillity::read_by_date_including_user(pool.get_ref(), date)
        .await
        .unwrap();

    let template = CalendarTemplate {
        user: user.into_inner(),
        date,
        areas,
        events,
        availabillities,
    };

    template.to_response()
}
